package flac

import (
	"testing"
	"time"
)

func TestGetMetadata(t *testing.T) {
	md, err := GetMetadata("test_files/audio.flac")
	if err != nil {
		t.Fatal(err)
	}
	expected := Metadata{
		SampleRate: 48000,
		BPS:        16,
		Channels:   2,
		NumSamples: 174564,
		MD5Sum:     "c4f2c7de2f0f952354bfec3a0809c00e",
	}
	if md != expected {
		t.Fatalf("%v != %v", md, expected)
	}
}

func TestLoad(t *testing.T) {
	l, r, err := Load("test_files/audio.flac")
	if err != nil {
		t.Fatal(err)
	}

	if err := Save("test_files/out.flac", l, r); err != nil {
		t.Fatal(err)
	}

	// Apparently necessary.
	time.Sleep(time.Second)

	md1, err := GetMetadata("test_files/audio.flac")
	if err != nil {
		t.Fatal(err)
	}

	md2, err := GetMetadata("test_files/out.flac")
	if err != nil {
		t.Fatal(err)
	}

	if md1 != md2 {
		t.Fatalf("%v != %v", md2, md1)
	}
}
